require 'acceptance_helpers'
require 'shared/examples'
require 'shared/contexts'

resource 'Collections' do
  header 'Accept', 'application/json'
  header 'Content-Type', 'application/json'

  before { 12.times { create(:collection); create(:learning_object) } }

  let(:collections) { Collection.all }
  let(:learning_objects) { LearningObject.all }
  
   get '/v1/collections' do
     parameter :limit, 'Limit of collections'
     parameter :offset, 'Offset of collections'
  
     let(:limit) { 12 }
     let(:offset) { 0 }
  
     example_request 'Get a list of collections' do
       expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(Collection.limit(limit).offset(offset).pluck(:id).sort)
       expect(status).to eq(200)
     end
   end

  get '/v1/collections/:id' do
    let(:id) { collections.first.id }

    example 'Get a collection' do
      do_request
      expect(path).to eq("/v1/collections/#{id}") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(Collection.find(id)))
      expect(status).to eq(200)
    end
  end

  delete '/v1/collections/:id' do
    include_context "authenticate_user"

    let(:id) { @collection.id }

    before do
      @collection = create(:collection, owner: @user)
    end

    example 'Destroying a collection' do
      do_request
      expect(status).to eq(200)
    end

  end

  post '/v1/collections/:id/items' do
    include_context "authenticate_user"
    
    parameter :items, 'Items belongs to the collection', scope: :collection

    let(:id) { @collection.id }
    let(:items) { [ { id: learning_objects.first.id, type: "LearningObject", position: "4" } ] }
    let(:raw_post) { params.to_json }

    before do
      @collection = create(:collection, owner: @user)
    end

    example 'Add items to collection' do
      do_request
      expect(status).to eq(200)
    end
  end

  delete '/v1/collections/:id/items' do
    include_context "authenticate_user"

    parameter :items, 'Items belongs to the collection', scope: :collection

    let(:id) { @collection.id }
    let(:items) { [ { id: @collection_item.id } ] }
    let(:raw_post) { params.to_json }

    before do
      @collection = create(:collection, owner: @user)
      @collection_item = create(:collection_item, collection: @collection)
    end

    example 'Remove items from collection' do
      do_request
      expect(status).to eq(200)
    end

  end


  post '/v1/collections/:id/like' do
    include_context "authenticate_user"

    let(:id) { collections.first.id }

    example 'Liking a collection' do
      likes = collections.first.likes.count
      do_request
      expect(path).to eq("/v1/collections/#{id}/like")
      expect(JSON.parse(response_body)['count']).to eq(likes + 1)
      expect(status).to eq(201)
    end
  end

  delete '/v1/collections/:id/like' do
    include_context "authenticate_user"

    let(:id) { collections.first.id }

    before do
      create(:like, user: @user, likeable: collections.first)
    end

    example 'Unliking a collection' do
      likes = collections.first.likes.count
      do_request
      expect(path).to eq("/v1/collections/#{id}/like")
      expect(JSON.parse(response_body)['count']).to eq(likes - 1)
      expect(status).to eq(200)
    end
  end

end
